/********************************************************************************
*                                                                               *
*                G L  C a n v a s   W i n d o w   W i d g e t                   *
*                                                                               *
*********************************************************************************
* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
*********************************************************************************
* $Id: FXGLCanvas.h,v 1.17 2000/09/19 15:25:28 jeroen Exp $                     *
********************************************************************************/
#ifndef FXGLCANVAS_H
#define FXGLCANVAS_H


/// GLCanvas, an area drawn by another object
class FXAPI FXGLCanvas : public FXCanvas {
  FXDECLARE(FXGLCanvas)
private:
  FXGLCanvas  *sgnext;  // Share group next in share list
  FXGLCanvas  *sgprev;  // Share group previous in share list
  void        *ctx;     // GL Context
protected:
  FXGLCanvas();
private:
  FXGLCanvas(const FXGLCanvas&);
  FXGLCanvas &operator=(const FXGLCanvas&);
#ifdef WIN32
  virtual const char* GetClass() const;
#endif
public:

  /**
  * Construct an OpenGL-capable canvas, with its own private display list.
  */
  FXGLCanvas(FXComposite* p,FXGLVisual *vis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /**
  * Construct an OpenGL-capable canvas, sharing display
  * list with another GL canvas.  This canvas becomes a member
  * of a display list share group.  All members of the display
  * list share group have to have the same visual.
  */
  FXGLCanvas(FXComposite* p,FXGLVisual *vis,FXGLCanvas* sharegroup,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /// Return TRUE if it is sharing display lists
  FXbool isShared() const;
  
  /// Create all of the server-side resources for this window 
  virtual void create();
  
  /// Detach the server-side resources for this window 
  virtual void detach();
  
  /// Destroy the server-side resources for this window 
  virtual void destroy();
  
  /// Make OpenGL context current prior to performing OpenGL commands
  virtual FXbool makeCurrent();
  
  /// Make OpenGL context non current
  virtual FXbool makeNonCurrent();
  
  /// Swap front and back buffer
  virtual void swapBuffers();
  
  /// Save object to stream
  virtual void save(FXStream& store) const;

  /// Load object from stream
  virtual void load(FXStream& store);

  /// Destructor
  virtual ~FXGLCanvas();
  };
  
  

#endif

